{ *********************************************************
  * LIVRE Delphi 7 Studio - De l'analyse  la ralisation *
  * Ed.EYROLLES, PARIS                                    *
  *********************************************************
  * Copyrights 2002 Olivier DAHAN:. - Paul TOTH           *
  *********************************************************
  * Le code fourni ici l'est uniquement  titre d'exemple *
  * Aucune "maintenance" ne sera fournie.                 *
  *********************************************************

}

{-----------------------------------------------------------------------------
 Unit Name: uatt2
 Author:    od
 Purpose:   Systme d'attente exemple pour livre Delphi 7
 History:   Mai 2002, V 1.0
-----------------------------------------------------------------------------}



unit uatt2;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, ExtCtrls, DateUtils;

type
  TFAttente = class(TForm)
    lmsg: TLabel;
    pbAttente: TProgressBar;
    lEstimation: TLabel;
    timEstimation: TTimer;
    procedure timEstimationTimer(Sender: TObject);
  private
    { Dclarations prives }
    FDepart : TDateTime; // heure de dpart
  public
    { Dclarations publiques }
  end;

  // Ouverture d'un message d'attente
  // Les appels peuvent tre superposs
  // Si WithGauge est Vrai, une jauge est affiche
  Procedure OpenWait(Const msg:string;WithGauge:boolean=false);

  // Fermeture du dernier message
  // Si CloseAll est Vrai, tous les messages empils
  // sont ferms
  Procedure CloseWait(CloseAll:boolean=False);

  // permet la mise  jour du message d'attente
  // ainsi que celle de la jauge
  Procedure SendWait(NewMsg : string;GaugeValue : integer = 0);


implementation
{$R *.dfm}
var
 MemoMsg : TStrings;  // Mmoire des messages "empilables"
 Fiche   : TFAttente; // la fiche d'attente

Procedure OpenWait(Const msg:string;WithGauge:boolean=false);
begin
 if assigned(Fiche) then
  // empilage du message et de la valeur de jauge
  MemoMsg.AddObject(Fiche.lmsg.Caption,tobject(Fiche.pbAttente.position)) else
  // sinon cration de la fiche
  Fiche := TFAttente.Create(Application);
 Fiche.lmsg.Caption := msg;              // le message d'attente
 Fiche.pbAttente.visible := WithGauge;   // afficher ou non la jauge
 Fiche.pbAttente.position := 0;          // raz de la jauge
 Fiche.FDepart := now;                   // heure de dpart
 Fiche.lEstimation.visible := WithGauge; // Estimation seulement si jauge prsente
 Fiche.timEstimation.Enabled:=WithGauge; // Enclenche le timer si jauge visible
 Fiche.Show;                             // afficher la fiche
 Fiche.Update;                           // mettre  jour l'affichage
end;

Procedure CloseWait(CloseAll:boolean=False);
begin
 if not assigned(Fiche) then exit;
 if closeAll then MemoMsg.Clear;
 if MemoMsg.Count>0 then // s'il existe des messages empils
  begin
   // rcupration du message au sommet de la pile
   Fiche.lmsg.Caption := MemoMsg[pred(MemoMsg.Count)];
   // rcupration de l'tat de la jauge
   Fiche.pbAttente.Visible := assigned(MemoMsg.Objects[pred(MemoMsg.Count)]);
   // rcupration de la position de la jauge
   Fiche.pbAttente.Position := integer(MemoMsg.Objects[pred(MemoMsg.Count)]);
   Fiche.Update; // mise  jour de la fiche
   MemoMsg.Delete(pred(MemoMsg.Count)); // suppression du message empil
  end else
  begin
   Fiche.Release;
   Fiche := nil;
  end;
end;

Procedure SendWait(NewMsg : string;GaugeValue : integer);
begin
 if not assigned(Fiche) then exit;
 if NewMsg<>'' then
  begin
   Fiche.lmsg.Caption:=NewMsg;
   Fiche.lmsg.Update;
  end;
 if GaugeValue in [0..100] then
  begin
   Fiche.pbAttente.Position := GaugeValue;
   Application.ProcessMessages;
  end;
end;

procedure TFAttente.timEstimationTimer(Sender: TObject);
var elapsed,estimation : double;
begin
 elapsed := MilliSecondSpan(Now,FDepart); // ms coules
 if pbAttente.Position=0 then estimation := 0 else
   estimation := (elapsed*(100.0-pbAttente.Position))/pbAttente.Position;
 lEstimation.visible := estimation>0;
 lEstimation.Caption := 'Temps restant: '+TimeToStr((estimation+1000)/msecsperday);
 lEstimation.Update;
end;

Initialization
MemoMsg := TStringlist.Create;
Fiche   := Nil;
Finalization
CloseWait(true);
FreeAndNil(MemoMsg);
end.
