{ *********************************************************
  * LIVRE Delphi 7 Studio - De l'analyse  la ralisation *
  * Ed.EYROLLES, PARIS                                    *
  *********************************************************
  * Copyrights 2002 Olivier DAHAN:. - Paul TOTH           *
  *********************************************************
  * Le code fourni ici l'est uniquement  titre d'exemple *
  * Aucune "maintenance" ne sera fournie.                 *
  *********************************************************

}

unit IBMainF;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  IBInstall, ComCtrls, StdCtrls, WinSvc;

type
  TMainForm = class(TForm)
    IBInstaller: TIBInstall;
    ProgressBar: TProgressBar;
    StatusBar: TStatusBar;
    InstallBtn: TButton;
    IBUnInstall1: TIBUnInstall;
    UninstallBtn: TButton;
    function IBInstallerStatusChange(Sender: TObject;
      StatusComment: String): TStatusResult;
    procedure InstallBtnClick(Sender: TObject);
    procedure UninstallBtnClick(Sender: TObject);
    function IBUnInstall1StatusChange(Sender: TObject;
      StatusComment: String): TStatusResult;
  private
    function GetIBRootDirectory: String;
    procedure StartInterbase;
    function GetErrorMessage: String;
    function GetIBServerDirectory: String;
    procedure ShutdownInterbase;
    function GetIBServiceHandle: SC_HANDLE;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

uses Registry, ShellAPI;

{$R *.DFM}

function TMainForm.GetErrorMessage: String;
{Returns the last Windows error message as an ANSI string.}
var
  ErrorCode:      DWORD;
  ErrorMsgStr:    array[0..255] of Char;
begin
  ErrorCode := GetLastError;
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM,
                nil, ErrorCode, 0, ErrorMsgStr, 256, nil);
  Result := StrPas(ErrorMsgStr);
end;

function TMainForm.GetIBServerDirectory: String;
{Returns the Interbase bin directory from the registry.}
var
  Reg:      TRegistry;
begin
  Reg := TRegistry.Create;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('\Software\Borland\Interbase\CurrentVersion', False) then
        Result := ReadString('ServerDirectory');
    end; //with
  finally
    Reg.Free;
  end; //try
end;

function TMainForm.GetIBServiceHandle: SC_HANDLE;
{Returns a handle to the Interbase server service under
 Windows NT/2000.}
var
  SCMHandle:           SC_HANDLE;
begin
  Result := 0;
  SCMHandle := OpenSCManager(nil, nil, GENERIC_EXECUTE);
  if SCMHandle = 0 then
    ShowMessage('SCM error: ' + GetErrorMessage)
  else begin
    Result := OpenService(SCMHandle, 'InterbaseGuardian', SERVICE_ALL_ACCESS);
    if Result = 0 then
      ShowMessage('Service open error: ' + GetErrorMessage)
  end; //if
end;

procedure TMainForm.StartInterbase;
var
  GuardianPath:        String;
  Result:              THandle;
  SCMHandle:           SC_HANDLE;
  ServiceHandle:       SC_HANDLE;
  ServiceArgs:         PAnsiChar;
begin
  ServiceArgs := nil;
  {If the platform is NT start the Interbase service otherwise
   execute the Interbase guardian.}
  if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
    ServiceHandle := GetIBServiceHandle;
    if ServiceHandle = 0 then Exit;
    if not StartService(ServiceHandle, 0, ServiceArgs) then
      ShowMessage('Service start error: ' + GetErrorMessage);
  end else begin
    {Execute the Interbase guardian on Win9x/Me.}
    GuardianPath := GetIBServerDirectory + 'ibguard.exe';
    Result := ShellExecute(Application.MainForm.Handle, nil,
                PChar(GuardianPath), '',
                PChar(GetIBRootDirectory), SW_NORMAL);
    if Result <= 32 then
      ShowMessage('Unable to start Interbase. Error = ' +
        IntToStr(Result));
  end; //if
end;

function TMainForm.GetIBRootDirectory: String;
{Returns the Interbase root directory from the registry.}
var
  Reg:      TRegistry;
begin
  Reg := TRegistry.Create;
  try
    with Reg do
    begin
      RootKey := HKEY_LOCAL_MACHINE;
      if OpenKey('\Software\Borland\Interbase\CurrentVersion', False) then
        Result := ReadString('RootDirectory');
    end; //with
  finally
    Reg.Free;
  end; //try
  Exit;
end;

function TMainForm.IBInstallerStatusChange(Sender: TObject;
  StatusComment: String): TStatusResult;
begin
  Result := srContinue;
  ProgressBar.Position := IBInstaller.Progress;
  StatusBar.SimpleText := StatusComment;
  Application.ProcessMessages;
end;

procedure TMainForm.InstallBtnClick(Sender: TObject);
begin
  with IBInstaller do
  begin
    SourceDirectory := ExtractFilePath(Application.ExeName);
    InstallCheck;
    InstallExecute;
    if RebootToComplete then
    begin
      if MessageDlg('You must restart Windows to complete the installation. Restart now?',
                    mtConfirmation, mbOkCancel, 0) = mrOK then
        ExitWindowsEx(EWX_REBOOT, 0)
      else
        ShowMessage('Remember to restart Windows to complete the installation.');
    end else begin
      StartInterbase;
      ShowMessage('Interbase has been installed');
    end; //if
    Application.Terminate;
  end; //with
end;

procedure TMainForm.ShutdownInterbase;
var
  ServiceStatus:       _SERVICE_STATUS;
  SCMHandle:           SC_HANDLE;
  ServiceHandle:       SC_HANDLE;
  GuardianWinHandle:   HWND;
  I:                   Integer;
begin
  if Win32Platform = VER_PLATFORM_WIN32_NT then
  begin
    ServiceHandle := GetIBServiceHandle;
    if ServiceHandle = 0 then
      Exit
    else begin
      {See if Interbase is running.}
      if not QueryServiceStatus(ServiceHandle, ServiceStatus) then
        ShowMessage('Query service error: ' + GetErrorMessage);
      if ServiceStatus.dwCurrentState <> SERVICE_STOPPED then
      begin
        if not ControlService(ServiceHandle, SERVICE_CONTROL_STOP, ServiceStatus) then
          ShowMessage('Service control error: ' + GetErrorMessage)
        else begin
          {Wait for the service to stop.}
          for I := 0 to 50 do
          begin
            QueryServiceStatus(ServiceHandle, ServiceStatus);
            if ServiceStatus.dwCurrentState = SERVICE_STOPPED then
              Break
            else
              Sleep(100);
          end; //for
        end; //if
      end; //if
    end; //if
  end else begin
    {If not running on NT/2000 then get the IB Guardian's window handle
     and send it a message to close.}
    GuardianWinHandle := FindWindow('IB_Guard', 'Interbase Guardian');
    if GuardianWinHandle <> 0 then
      SendMessage(GuardianWinHandle, WM_CLOSE, 0, 0);
  end; //if
end;

procedure TMainForm.UninstallBtnClick(Sender: TObject);
begin
  ShutdownInterbase;
  with IBUninstall1 do
  begin
    UninstallFile := GetIBRootDirectory + 'ibuninst.000';
    UninstallCheck;
    UninstallExecute;
    ShowMessage('Interbase has been removed.');
    Application.Terminate;
  end; //with
end;

function TMainForm.IBUnInstall1StatusChange(Sender: TObject;
  StatusComment: String): TStatusResult;
begin
  Result := srContinue;
  ProgressBar.Position := IBInstaller.Progress;
  StatusBar.SimpleText := StatusComment;
  Application.ProcessMessages;
end;

end.
