{ *********************************************************
  * LIVRE Delphi 7 Studio - De l'analyse  la ralisation *
  * Ed.EYROLLES, PARIS                                    *
  *********************************************************
  * Copyrights 2002 Olivier DAHAN:. - Paul TOTH           *
  *********************************************************
  * Le code fourni ici l'est uniquement  titre d'exemple *
  * Aucune "maintenance" ne sera fournie.                 *
  *********************************************************

}

program wServer;

{$APPTYPE CONSOLE}

uses
  Windows,Winsock,Classes,SysUtils;

procedure FatalError(msg:string);
 begin
  WriteLn(msg);
  ReadLn;
  halt;
 end;

var
 wsaData :TWSAData;
 server  :TSocket;
 SockAddr:TSockAddr;
 Lock    :TRTLCriticalSection;
 Clients :TList;
 client  :TSocket;
 SockSize:integer;
 ClientCount:integer;

Type
 TClient=class
  Handle :TSocket; // le socket client
  Address:string;  // adresse IP du client
  Thread :THandle; // Thread de communication
  Buffer :string;  // tampon de lecture du socket
  Constructor Create;
  Destructor Destroy; override;
  procedure Exec;
  Function ReadLine:string;
  procedure SendLine(s:string);
 end;

function ClientThread(Sender:TClient):integer; stdcall;
 begin
  Result:=0;
  Try
   Sender.Exec;
  Finally
   Sender.Free;
  end;
 end;

procedure Dispath(s:string);
 var
  i:integer;
 begin
  s:=s+#13#10;
  EnterCriticalSection(Lock);
  try
   for i:=0 to Clients.Count-1 do begin
    TClient(Clients[i]).SendLine(s);
   end;
  finally
   LeaveCriticalSection(Lock);
  end;
 end;

Constructor TClient.Create;
 var
  id:Cardinal;
 begin
 // on conserve le numro de socket
  Handle:=Client;
 // on rcupre son adresse IP
  Address:=inet_ntoa(SockAddr.sin_addr);
 // ajout dans la liste des clients
  EnterCriticalSection(Lock);
  try
   Clients.Add(Self);
   writeln(address,' : nouvelle connection - ',Clients.Count,' client(s)');
  finally
   LeaveCriticalSection(Lock);
  end;
 // lancement d'un Thread client
  Thread:=CreateThread(nil,0,@ClientThread,Self,0,id);
 end;

Destructor TClient.Destroy;
 begin
  shutdown(Handle,SD_BOTH);
  closesocket(Handle);
  EnterCriticalSection(Lock);
  try
   Clients.Remove(Self);
  finally
   LeaveCriticalSection(Lock);
  end;
  CloseHandle(Thread);
 end;

Procedure TClient.Exec;
 var
  s:string;
 begin
  try
   if ReadLine<>'GET /wChat HTTP 1.0' then exit; // client invalide
   if ReadLine<>'' then exit; // client invalide
  // envoyer une rponse de type HTTP
   SendLine('HTTP/1.0 200 OK'#13#10);
   SendLine('Content-type: text/plain'#13#10#13#10);
  // lecture des messages
   s:=ReadLine;
   while s<>'' do begin
    Dispath(s);
    s:=ReadLine;
   end;
  except
   on e:Exception do begin
    EnterCriticalSection(Lock);
    try
     WriteLn(Address,' : ',e.Message);
    finally
     LeaveCriticalSection(Lock);
    end;
   end;
  end;
 end;

Function TClient.ReadLine:string;
 var
  temp:string;
  i:integer;
 begin
// lecture du socket
  i:=pos(#13#10,Buffer);
  while i=0 do begin
   SetLength(temp,1024);
   i:=recv(Handle,Temp[1],1024,0);
   if i<=0 then raise Exception.Create('fin de communication');
   Buffer:=Buffer+Copy(Temp,1,i);
   i:=pos(#13#10,Buffer);
  // limiter la taille du buffer interne !
   if (i=0)and(Length(Buffer)>80) then raise Exception.Create('client trop bavard !');
  end;
  Result:=Copy(Buffer,1,i-1);
  Delete(Buffer,1,i+1);
 end;

procedure TClient.SendLine(s:string);
 begin
  send(Handle,s[1],length(s),0);
 end;

begin
 WriteLn('WebChat serveur v1.0.');
 WriteLn('Ctrl+C pour fermer');

// Initialisation de WINSOCK
 if wsaStartup($101,wsaData)<>0 then FatalError('Erreur winsock');

// allocation d'un socket TCP
 server:=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
 if server=INVALID_SOCKET then FatalError('Impossible d''allouer un socket');

// dclarer le port 80
 FillChar(SockAddr,SizeOf(SockAddr),0);
 SockAddr.sa_family:=AF_INET;
 SockAddr.sin_port :=htons(80); // HTTP
 if bind(server,SockAddr,SizeOf(SockAddr))<>0 then FatalError('Impossible de dmarrer le service 80');

// coute les clients
 if listen(server,0)<>0 then FatalError('Dmarrage du serveur impossible');

// la liste des client et son verrou
 Clients:=TList.Create;
 InitializeCriticalSection(Lock);

// boucle principale
 SockSize:=SizeOf(SockAddr);
 client:=accept(server,@SockAddr,@SockSize);
 while client<>INVALID_SOCKET do begin
  TClient.Create;
  client:=accept(server,@SockAddr,@SockSize);
 end;

// fermeture du socket serveur (implique la dconnexion de tous les clients)
 shutdown(server,SD_BOTH);
 closesocket(server);

// attend la fermeture des connexions clientes
 repeat
  Sleep(1); // pour ne pas saturer la machine !
  EnterCriticalSection(Lock);
  try
   ClientCount:=Clients.Count;
  finally
   LeaveCriticalSection(Lock);
  end;
 until ClientCount=0;
 DeleteCriticalSection(Lock);

// Finalisation de WINSOCK
 wsaCleanup;
 writeln('Done');
 ReadLn;
end.
