{ *********************************************************
  * LIVRE Delphi 7 Studio - De l'analyse  la ralisation *
  * Ed.EYROLLES, PARIS                                    *
  *********************************************************
  * Copyrights 2002 Olivier DAHAN:. - Paul TOTH           *
  *********************************************************
  * Le code fourni ici l'est uniquement  titre d'exemple *
  * Aucune "maintenance" ne sera fournie.                 *
  *********************************************************

}

{-----------------------------------------------------------------------------
 Unit Name: uObserver
 Author:    od
 Purpose:   Design Pattern "Observer", example de mise en oeuvre
            simulation d'une gestion de stock minimaliste
 History:
-----------------------------------------------------------------------------}

unit uObserver;

interface
uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs;

Type

 TObservedStock = class; // dclaration forward

 // type d'observation possible. Tout, entres en stock, sorties, ou stock <=0
 TObservationType  = (ot_all,ot_Input,ot_Output,ot_NullStock);
 // type du callback qui sera enregistr par chaque observateur
 TObservationEvent = Procedure (Observed:TObservedStock;Event:TObservationType) of object;
 // type interne de stockage d'un observateur
 TObservation      = class
                      obs : TObservationEvent; // le callback enregistr
                      kind: TObservationType;  // le type d'observation dsire
                     end;
 // La classe de gestion de stock qui implmente le mcanisme
 // lui permettant d'tre observe
 TObservedStock = class
  private
   fIn, fOut : integer;
   fObservers: Tlist;
   procedure SetInput(value:integer);
   procedure SetOutput(value:integer);
   function  GetStock : integer;
   function  GetObserverIndex(obs:TObservationEvent):integer;
   procedure NotifyObservers(kind:TObservationType);
  public
   constructor Create; virtual;
   destructor  Destroy; override;

   // enregistrement d'un observateur
   procedure RegisterObserver(Callback:TObservationEvent;EventKind:TObservationType);
   // dsenregistrement d'un observateur
   procedure UnregisterObserver(callback:TObservationEvent);

   // mise  zro de l'objet stock
   procedure ResetStock(Notify:boolean=true);

   // lecture du stock courant
   property Stock : integer read getStock;
   // lecture du total des entres, et saisie d'une entre en stock
   property Input : integer read fIn write setInput;
   // lecture du total des sorties, et saisie d'une sortie de stock
   property Output: integer read fOut write SetOutput;
  end;

implementation

constructor TObservedStock.Create;
begin
 inherited create;
 fObservers := Tlist.Create; // liste de pointeurs de TObservation
end;

destructor  TObservedStock.Destroy;
var i:integer;
begin
 for i:=0 to fObservers.count-1 do
  TObservation(fObservers[i]).Free;
 fObservers.Free;
 inherited destroy;
end;


procedure TObservedStock.SetInput(value:integer);
begin
 // saisir une entre en stock
 inc(fIn,value);
 // notifier les observateurs intresss par les entres en stock
 NotifyObservers(ot_Input);
end;

procedure TObservedStock.SetOutput(value:integer);
begin
 // saisir une sortie de stock
 inc(fOut,value);
 // notifier les observateurs intresss par les sorties de stock
 NotifyObservers(ot_Output);
 // si stock<=0 notifier les observateurs s'intressant  ce cas
 if GetStock<1 then NotifyObservers(ot_NullStock);
end;

function  TObservedStock.GetStock : integer;
begin
 // lecture de l'tat du stock
 result := fIn-fOut;
end;

// mthode interne retournant l'index d'un observateur dans la liste
function  TObservedStock.GetObserverIndex(obs:TObservationEvent):integer;
var i:integer;
begin
 result:=-1;
 for i:=0 to fObservers.Count-1 do
  if (Tmethod(TObservation(fObservers).obs).Code =Tmethod(obs).Code) and
     (Tmethod(TObservation(fObservers).obs).Data =Tmethod(obs).Data) then
   begin
    result:=i;
    break;
   end;
end;

// enregistrer un observateur et le type d'vnement observ
procedure TObservedStock.RegisterObserver(Callback:TObservationEvent;EventKind:TObservationType);
var i:integer; o:TObservation;
begin
 i:= GetObserverIndex(Callback);
 if i=-1 then
  begin
   o:=tobservation.create;
   o.obs:=Callback;
   o.kind := EventKind;
   fObservers.add(o);
  end;
end;

// dsenregistrer un observateur
procedure TObservedStock.UnregisterObserver(callback:TObservationEvent);
var i:integer;
begin
 i := GetObserverIndex(Callback);
 if i>-1 then
  begin
   TObservation(fObservers[i]).Free;
   fObservers.Delete(i);
  end;
end;

// mise  zro des entres et sorties
procedure TObservedStock.ResetStock(Notify:boolean=true);
begin
 fIn:=0;
 fOut:=0;
 if Notify then
  begin
   NotifyObservers(ot_Input);
   NotifyObservers(ot_Output);
   NotifyObservers(ot_NullStock);
  end;
end;

procedure TObservedStock.NotifyObservers(kind:TObservationType);
var i:integer;
begin
 // notification de tous les observateurs intresss par le
 // type d'vnement spcifi par Kind
 for i:=0 to fObservers.count-1 do
  if TObservation(fObservers[i]).kind in [ot_all,kind] then
    TObservation(fObservers[i]).obs(self,kind);
end;

end.
